from keras.models import Sequential, Model
from keras.layers.embeddings import Embedding
from keras.layers.core import Activation, Dense, Merge, Permute, Flatten, Dropout, TimeDistributedDense, Reshape, Layer, \
    ActivityRegularization, RepeatVector
from keras.layers.recurrent import LSTM
from keras.layers.convolutional import Convolution2D, MaxPooling2D, Convolution1D, MaxPooling1D, AveragePooling1D
from keras.preprocessing.sequence import pad_sequences
from keras.layers.wrappers import TimeDistributed
from keras.callbacks import History
from keras.layers import Input, Dense, Embedding, merge, Dropout
import numpy as np
from data_helper import load_data
from result_calculator import *
from keras import backend as K
#from sklearn.metrics import average_precision_score, precision_score, recall_score
from keras.utils.visualize_util import plot
import cPickle
from keras.utils import np_utils
import h5py

if __name__ == '__main__':
    print ('loading data...')
    x, img_x, y, valid_x, valid_img_x, valid_y, test_x, test_img_x, test_y, vocabulary, vocabulary_inv, hashtagVoc, hashtagVoc_inv, maxlen = load_data()

    vocab_size = len(vocabulary_inv) + 1
    hashtag_size = len(hashtagVoc_inv)

    print('-')
    print('Vocab size:', vocab_size, 'unique words')
    print('Hashtag size:', hashtag_size, 'unique hashtag')
    print('Max length:', maxlen, 'words')
    print('-')
    print('Here\'s what a "hashtag" tuple looks like (x, img_x, label):')
    print(x[0], img_x[0].shape, y[0])
    print('-')
    print('-')
    print('input_mention_tweet: integer tensor of shape (samples, max_length)')
    print('shape:', x.shape)
    print('-')
    print('-')
    print('input_label: integer tensor of shape (samples, hashtag_size)')
    print('shape:', y.shape)
    print('-')
    embedding_dim =100
    nb_epoch = 20
    batch_size = 1000

    feat_dim = 512
    w = 7
    num_region = 49

    # build model
    print "Build model..."

    tweet = Input(shape=(maxlen,), dtype='int32')
    embedding = Embedding(input_dim=vocab_size, output_dim=embedding_dim, input_length=maxlen, mask_zero=False)(
        tweet)
    lstm = LSTM(embedding_dim, return_sequences=True, input_shape=(maxlen, embedding_dim))(embedding)
    dropout = Dropout(0.5)(lstm)

    img = Input(shape=(1,feat_dim, w, w))
    img_reshape = Reshape((feat_dim, w * w))(img)
    img_permute = Permute((2, 1))(img_reshape)

    # text->img
    img_dense = TimeDistributed(Dense(embedding_dim))(img_permute)
    tweet_avg = AveragePooling1D(pool_length=maxlen)(dropout)
    tweet_avg = Flatten()(tweet_avg)
    tweet_avg_dense = Dense(embedding_dim)(tweet_avg)
    tweet_repeat = RepeatVector(w * w)(tweet_avg_dense)
    att_1 = merge([tweet_repeat, img_dense], mode='concat')
    att_1 = Activation('tanh')(att_1)
    att_1 = TimeDistributed(Dense(1))(att_1)
    att_1 = Activation('softmax')(att_1)
    att_1_pro = Flatten()(att_1)

    img_new = merge([att_1_pro, img_dense], mode='dot', dot_axes=(1, 1))

    # img->text
    img_new_dense = Dense(embedding_dim)(img_new)
    img_new_repeat = RepeatVector(maxlen)(img_new_dense)
    tweet_dense = TimeDistributed((Dense(embedding_dim)))(dropout)
    att_2 = merge([img_new_repeat, tweet_dense], mode='concat')
    att_2 = Activation('tanh')(att_2)
    att_2 = TimeDistributed(Dense(1))(att_2)
    att_2 = Activation('softmax')(att_2)
    att_2_pro = Flatten()(att_2)

    tweet_new = merge([att_2_pro, dropout], mode='dot', dot_axes=(1, 1))

    # set gate
    img_new_tanh = Dense(embedding_dim, activation='tanh')(img_new)
    tweet_new_tanh = Dense(embedding_dim, activation='tanh')(tweet_new)
    merge = merge([img_new_tanh, tweet_new_tanh], mode='concat')
    z = Dense(1, activation='sigmoid')(merge)

    output = Dense(hashtag_size, activation='softmax')(merge)
    model = Model(input=[tweet, img], output=output)
    model.compile(optimizer='adam', loss='categorical_crossentropy', metrics=["accuracy"])
    print model.summary()
    print "finished building model"
    plot(model, show_shapes=True, to_file='model.png')

    # starts training
    y = np_utils.to_categorical(y, hashtag_size)
    print "starts training"
    best_f1 = 0
    topK = [1, 2, 3, 4, 5]
    for j in xrange(nb_epoch):
        history = History()
        model.fit([x, img_x], y, batch_size=batch_size, nb_epoch=1, verbose=1, callbacks=[history])

        print history.history
        print len(history.history)
        y_pred = model.predict([valid_x, valid_img_x], batch_size=batch_size, verbose=0)

        y_pred = np.argsort(y_pred, axis=1)
        precision = precision_score(valid_y, y_pred)
        recall = recall_score(valid_y, y_pred)
        F1 = 2 * (precision * recall) / (precision + recall)
        print "Epoch:", (j + 1), "precision:", precision, "recall:", recall, "f1 score:", F1

        if best_f1 < F1:
            best_f1 = F1
            y_pred = model.predict([test_x, test_img_x], batch_size=batch_size, verbose=0)

            y_pred = np.argsort(y_pred, axis=1)
            for k in topK:
                precision = precision_score(test_y, y_pred, k=k)
                recall = recall_score(test_y, y_pred, k=k)
                hscore = hits_score(test_y, y_pred, k=k)
                F1 = 2 * (precision * recall) / (precision + recall)
                print "\t", "top:", k, "Test: precision:", precision, "recall:", recall, "f1 score:", F1, "hits score:", hscore

