# -*- coding: utf-8 -*-
import numpy as np
from collections import Counter
import itertools
import os
import cPickle
import h5py
from keras.preprocessing.sequence import pad_sequences

trainFile = "train.txt"
validFile = "valid.txt"
testFile = "test.txt"
dataPath = "../data/"
h5_file = "Train_feat.h5"
f = h5py.File(os.path.join(dataPath, h5_file), 'r')


def load_data():
    maxlen = 0
    x, img_x, y, maxlen = load_traindata(maxlen, trainFile)
    valid_x, valid_img_x, valid_y, maxlen = load_testdata(maxlen, validFile)
    test_x, test_img_x, test_y, maxlen = load_testdata(maxlen, testFile)

    print "Train set size: ", len(x), len(img_x), len(y)
    print "Valid set size: ", len(valid_x), len(valid_img_x), len(valid_y)
    print "Test set size: ", len(test_x), len(test_img_x), len(test_y)
    print "Max word len: ", maxlen

    vocabulary, vocabulary_inv, hashtagVoc, hashtagVoc_inv = build_vocab(x, y, valid_x, valid_y, test_x, test_y)
    x, img_x, y = build_input_data(x, img_x, y, vocabulary, hashtagVoc, maxlen)
    valid_x, valid_img_x, valid_y = build_test_data(valid_x, valid_img_x, valid_y, vocabulary, hashtagVoc, maxlen)
    test_x, test_img_x, test_y = build_test_data(test_x, test_img_x, test_y, vocabulary, hashtagVoc, maxlen)
    '''
    print x[0],img_x[0],y[0]
    print valid_x[0],valid_img_x[0],valid_y[0]
    print test_x[0],test_img_x[0],test_y[0]
    '''

    return [x, img_x, y, valid_x, valid_img_x, valid_y, test_x, test_img_x, test_y, vocabulary, vocabulary_inv,
            hashtagVoc, hashtagVoc_inv, maxlen]


def build_input_data(x, img_x, y, vocabulary, hashtagVoc, maxlen):
    x = np.asarray([[vocabulary[w] for w in t if w in vocabulary] for t in x])
    x = pad_sequences(x, maxlen=maxlen).astype(np.int32)
    img_x = np.asarray(img_x)
    y = np.asarray([hashtagVoc[h] for h in y], dtype=np.int32)

    return [x, img_x, y]


def build_test_data(x, img_x, y, vocabulary, hashtagVoc, maxlen):
    x = np.asarray([[vocabulary[w] for w in t if w in vocabulary] for t in x])
    x = pad_sequences(x, maxlen=maxlen).astype(np.int32)
    img_x = np.asarray(img_x)
    y = np.asarray([[hashtagVoc[h] for h in hlist] for hlist in y])

    return [x, img_x, y]


def build_vocab(x, y, valid_x, valid_y, test_x, test_y):
    vocabFileName = 'vocabulary_full.pkl'

    if os.path.isfile(vocabFileName):
        print "loading vocabulary..."
        vocabulary, vocabulary_inv, hashtagVoc, hashtagVoc_inv = cPickle.load(open(vocabFileName))
        vocabulary_inv = vocabulary_inv[:500000]
        vocabulary = {x: i + 1 for i, x in enumerate(vocabulary_inv)}
    else:
        print "calculating vocabulary..."
        text = []
        for x_t in [x, valid_x, test_x]:
            for t in x_t:
                for w in t:
                    text.append(w)

        print len(text)
        word_counts = Counter(text)

        vocabulary_inv = [x[0] for x in word_counts.most_common()]
        vocabulary = {x: i + 1 for i, x in enumerate(vocabulary_inv)}

        hashtaglist = []
        for h in y:
            hashtaglist.append(h)
        for hlist in valid_y + test_y:
            for h in hlist:
                hashtaglist.append(h)

        hashtag_count = Counter(hashtaglist)
        hashtagVoc_inv = [x[0] for x in hashtag_count.most_common()]
        hashtagVoc = {x: i for i, x in enumerate(hashtagVoc_inv)}

        cPickle.dump([vocabulary, vocabulary_inv, hashtagVoc, hashtagVoc_inv], open(vocabFileName, "w"))
    return [vocabulary, vocabulary_inv, hashtagVoc, hashtagVoc_inv]


def load_testdata(maxlen, Filename):
    print Filename
    x = []
    img_x = []
    y = []
    index = 0
    infile = open(dataPath + Filename)
    for line in infile:
        line = line.strip().split('\t')
        hashtagList = line[2].split('||')
        tweet = line[1].split(' ')
        tweet = [w.rstrip() for w in tweet if w != '']
        maxlen = max(len(tweet), maxlen)
        x.append(tweet)
        y.append(hashtagList)
        # image matrix store in list
        data = f.get(line[0])
        np_data = np.array(data)
        img_x.append(np_data)
    # img_x.append(line[0])
    return x, img_x, y, maxlen


def load_traindata(maxlen, Filename):
    print Filename
    x = []
    img_x = []
    y = []
    index=0
    cur_hashtag_count = 0
    infile = open(dataPath + Filename)
    for line in infile:
        index+=1
        if index%100000==0:
            print index
        line = line.strip().split('\t')
        hashtagList = line[2].split('||')
        cur_hashtag_count = len(hashtagList)
        tweet = line[1].split(' ')
        tweet = [w.rstrip() for w in tweet if w != '']
        maxlen = max(len(tweet), maxlen)
        for i in xrange(cur_hashtag_count):
            x.append(tweet)
        for h in hashtagList:
            y.append(h)
        for i in xrange(cur_hashtag_count):
            data = f.get(line[0])
            np_data = np.array(data)
            img_x.append(np_data)
    # img_x.append(line[0])
    return x, img_x, y, maxlen


if __name__ == '__main__':
    load_data()
    f.close()
